﻿package {
	
	
	/**
	 * ...
	 * @author necoEngine
	 */
	public class RamdomTable {
		
		private var randomRange:uint;           // 抽選テーブルのスポットの数
		private var rangeRate:Number;           // 抽選テーブルスレート
		private var numCounters:uint;           // 目の数
		private var counters:Vector.<uint>;     // 目の出現回数カウンタ。目の数=要素数
		private var randomTable:Vector.<uint>;  // 抽選テーブル。
		private var indexesTable:Vector.<uint>; // 抽選テーブルを作成する為のくじ引き用配列
		
		
		
		public function RamdomTable(num:uint, range:uint) {
			
			numCounters = num;
			randomRange = range;
			rangeRate   = randomRange * 0.01;
			
			init();
			
		} // constructor close
		
		
		private function init():void {
			
			var i:uint;
			
			// カウンタに初期値の1を入れておく。0だと計算が成立しない。
			counters = new Vector.<uint>(numCounters, true);
			for (i = 0; i < numCounters; i++) counters[i] = 1;
			
			// 抽選テーブル配列初期化
			randomTable = new Vector.<uint>(randomRange, true);
			
			// くじ引き用配列の用意。抽選テーブルのインデックスを保持。
			indexesTable = new Vector.<uint>(randomRange);
			for (i = 0; i < randomRange; i++) indexesTable[i] = i;
			
			// 抽選テーブル作成
			for (i = 0; i < numCounters; i++) tableUpdate(i, Math.floor(counters[i] / numCounters * 100));
			
		} // init() close
		
		
		private function tableUpdate(index:uint, rate:int):void {
			/**
			 * 機能：抽選テーブルの分布を更新する。
			 * 引数：第1引数 目の番号
			 *      第2引数 抽選テーブル中における目の分布量
			 * 戻り値：なし
			 * 備考：
			 */
			
			var target:uint, tableIndex:int, len:uint, cellNum:uint, cellCount:uint = 0;
			len = indexesTable.length;
			cellNum = Math.floor(rate * rangeRate);
			
			while (cellCount < cellNum) {
				target = Math.floor(Math.random() * len); // くじ引き配列のインデックス
				tableIndex = indexesTable[target];        // くじ引き結果
				indexesTable.splice(target, 1);           // 引いたくじを捨てる
				len = indexesTable.length;                // くじが減るので、配列長を更新
				
				randomTable[tableIndex] = index; // くじ引き結果をテーブルの要素に反映
				cellCount++;
			}
			
		} // tableUpdate() close
		
		
		private function counterRegulator(index:uint):void {
			/**
			 * 機能：出目に応じて各目のカウンタを上昇させ、
			 *      それに合わせて次の抽選テーブルにおける目の配分を算出し、次の抽選テーブルを用意する
			 * 引数：出目
			 * 戻り値：なし
			 * 備考：抽選テーブルには初期値として必ず0を入れる為、先頭の目（インデックス0の要素）が少し得。
			 */
			
			var i:uint, total:Number = 0, countersBuff:Vector.<Number>;
			
			counters[index]++; // 出た目のカウンタを上げる
			
			// 全ての目の出現回数を合計する
			for (i = 0; i < numCounters; i++) total += counters[i];
			
			// 合計した値を各目の出現回数の値で除算。（小数点第2位に丸める）
			countersBuff = new Vector.<Number>(numCounters);
			for (i = 0; i < numCounters; i++) countersBuff[i] = Math.floor(total / counters[i] * 100) * 0.01;
			
			//除算の解を再び合計する。
			total = 0;
			for (i = 0; i < numCounters; i++) total += countersBuff[i];
			
			// 抽選テーブル配列初期化（最初は全て0が入る）
			randomTable = null;
			randomTable = new Vector.<uint>(randomRange, true);
			
			// くじ引き用配列の用意。抽選テーブルのインデックスを保持。
			indexesTable = null;
			indexesTable = new Vector.<uint>(randomRange);
			for (i = 0; i < randomRange; i++) indexesTable[i] = i;
			
			// 抽選テーブル作成
			for (i = 0; i < numCounters; i++) tableUpdate(i, Math.floor(countersBuff[i] / total * 100));
			countersBuff = null;
			
		} // counterRegulator() close
		
		
		
//================================================================================
// Public
//================================================================================
		public function getIndex():uint {
			/**
			 * 機能：抽選テーブルから値を取り出して返し、counterRegulator()を実行する。
			 * 引数：なし
			 * 戻り値：出目番号
			 * 備考：
			 */
			
			var index:uint = randomTable[Math.floor(Math.random() * randomRange)];
			counterRegulator(index);
			
			return index;
			
		} // getIndex() close
		
		
		public function reset():void {
			/**
			 * 機能：値のリセットを行う。リファレンス型にnullを入れてからinit()を実行
			 * 引数：なし
			 * 戻り値：なし
			 * 備考：
			 */
			
			counters     = null;
			randomTable  = null;
			indexesTable = null;
			init();
			
		} // reset() close
	}
}